/*
  Dingoo minimal library by Slaanesh
  Based on Rlyeh's minimal.c for GP2X
*/

#define KEYBOARD 1

static unsigned long		dingoo_dev_fb[2] = {0,0};
//static unsigned long		dingoo_dev_dsp = 0;
//static unsigned long		dingoo_dev_mixer = 0;
static unsigned long		dingoo_dev_gpio = 0;
static unsigned long		dingoo_dev_backlight = 0;
static volatile unsigned long	*dingoo_gpioregl;


unsigned char                   *dingoo_screen8;
unsigned short                  *dingoo_screen15;
unsigned int                    *dingoo_screen32;
//int                             dingoo_sound_rate=16000;
//int                             dingoo_sound_stereo=0;
int                             rotate_controls=0;
unsigned short			dingoo_palette[256];

#include "minimal.h"
//#include "dingoo_sound.cpp"
//#include "dingoo_cpu.cpp"
#include "dingoo_keyboard.cpp"

void dingoo_video_flip(void)
{
}


void dingoo_video_flip_single(void)
{
}


void dingoo_video_setpalette(void)
{
}

void dingoo_flush_video(void)
{
	// This actually flushes the entire cache in both DCACHE, ICACHE.
	//
	cacheflush((char *)dingoo_screen15,  DINGOO_SCREEN_WIDTH * DINGOO_SCREEN_HEIGHT * sizeof(unsigned short), DCACHE);
}


unsigned long dingoo_joystick_read(unsigned int use_virtual)
{
	/* Read GPIO registers */
	unsigned long gpio2 = dingoo_gpioregl[0x200>>2];
	unsigned long gpio3 = dingoo_gpioregl[0x300>>2];

	/* gpio pxpin is active low, so flip bits */
	gpio2 ^= 0xffffffff;
	gpio3 ^= 0xffffffff;

	/* mask the bits that we want */
	gpio2 &= MASK_GPIO2;
	gpio3 &= MASK_GPIO3;

	/* shift gpio2 >> 1 as START and SELECT occupy the same bit */
	gpio2 >>= 1;

	/* merge together to make one bit field */
	gpio3 |= gpio2;

	/* If using virtual keys, check whether SELECT is being pressed */
	if (use_virtual && (gpio3 & DINGOO_SELECT))
	{
		gpio3 |= (gpio3 & MASK_SEL_SHIFT)<<7;
		gpio3 &= ~DINGOO_SELECT;	/* remove SELECT */
	}
	return gpio3;
}

void dingoo_joystick_press (void)
{
	while(!dingoo_joystick_read(1));
}



void dingoo_timer_delay(clock_t milliseconds)
{
	clock_t now=dingoo_timer_read();
	while (dingoo_timer_read()-now<(milliseconds*1000));
}


clock_t dingoo_timer_read(void)
{
	return clock(); /* CLOCKS_PER_SEC = 1000000 */
}

void dingoo_timer_profile(void)
{
}

//void dingoo_init(int rate, int bits, int stereo, unsigned int mhz, unsigned int brightness)
void dingoo_init(void)
{
//	int frag;

#if KEYBOARD
	OpenKeyboard();
	EnterGraphicsMode();
#endif

        dingoo_dev_fb[0] = open("/dev/fb0",   O_RDWR);
        //dingoo_dev_dsp = open("/dev/dsp",   O_WRONLY);
        //dingoo_dev_mixer = open("/dev/mixer", O_WRONLY);

	dingoo_dev_gpio = open("/dev/mem", O_RDWR);
	dingoo_dev_backlight = open("/proc/jz/lcd_backlight", O_RDWR);
        dingoo_gpioregl=(unsigned long  *)mmap(0, 0x300, PROT_READ|PROT_WRITE, MAP_SHARED, dingoo_dev_gpio, 0x10010000);

	/* map framebuffer to write to video */
        dingoo_screen15=(unsigned short *)mmap(0, DINGOO_SCREEN_WIDTH * DINGOO_SCREEN_HEIGHT * sizeof(unsigned short), PROT_WRITE, MAP_SHARED, dingoo_dev_fb[0], 0);
	if( dingoo_screen15==MAP_FAILED )
	{
		printf("mmap for dingoo_screen15 failed\n");
		exit(1);
	}
	dingoo_screen8=(unsigned char *)dingoo_screen15; /* no 8-bit screen */
	dingoo_screen32=(unsigned int *)dingoo_screen15;

	/* clear screen */
	dingoo_clear_video();

	/* sound init */
	//ioctl(dingoo_dev_dsp, SNDCTL_DSP_SETFMT, &bits);
	//ioctl(dingoo_dev_dsp, SNDCTL_DSP_STEREO, &stereo);
	//ioctl(dingoo_dev_dsp, SNDCTL_DSP_SPEED,  &rate);

	/* Setting fragsize. Unfortunately minimum frag size with
	 * the current driver is 4 * PAGESIZE or 16KB
	 */
	//frag = 0x00100007;	/* 16 Buffers at 128bytes each, see above */
	//ioctl(dingoo_dev_dsp, SNDCTL_DSP_SETFRAGMENT, &frag);

	/* sound volume */
	//dingoo_sound_volume(40,40);

	/* set dingoo cpu speed */
	//if (mhz != DINGOO_DEFAULT_MHZ)
	//{
	//	dingoo_set_clock(mhz);
	//}

	/* set dingoo screen brightness */
	//dingoo_set_brightness(brightness);
}

#if defined(__cplusplus)
extern "C" {
#endif
extern int fcloseall(void);
#if defined(__cplusplus)
}
#endif

void dingoo_deinit(void)
{
	//dingoo_set_clock(336);	/* default speed */
	
	dingoo_clear_video();

#if KEYBOARD
	CloseKeyboard();
	LeaveGraphicsMode();
#endif
	
	/* close devices */
	/* unmap framebuffer */
	munmap ((void *)dingoo_screen15, DINGOO_SCREEN_WIDTH * DINGOO_SCREEN_HEIGHT * sizeof(unsigned short));

	munmap((void *)dingoo_gpioregl, 0x300);
	//close(dingoo_dev_mixer);
	//close(dingoo_dev_dsp);
	close(dingoo_dev_fb[0]);
	close(dingoo_dev_backlight);
	fcloseall();
}

void dingoo_set_video_mode(int bpp,int width,int height)
{
	dingoo_clear_video();

	/*
	 * Not much else to do here as we're are always using
	 * a 320 x 240 x 16bit screen
	 */
}

void dingoo_video_wait_vsync(void)
{
	/* Looks like this is impossible with the Dingoo's LCD */
	/* TV out uses the built in controller so it may be useful for that */
}

void dingoo_clear_video(void)
{
	// Clear to black
	memset(dingoo_screen15, 0x0, DINGOO_SCREEN_WIDTH * DINGOO_SCREEN_HEIGHT *sizeof(unsigned short));
}
